@@ -1,9 +1,9 @@
-# $Id: Composers.pm 2186 2007-03-06 19:20:58Z comdog $
+# $Id: Composers.pm 2251 2007-04-16 21:02:34Z comdog $
 package Brick::Composers;
 use base qw(Exporter);
 use vars qw($VERSION);
 
-$VERSION = sprintf "1.%04d", q$Revision: 2186 $ =~ m/ (\d+) /xg;
+$VERSION = sprintf "1.%04d", q$Revision: 2251 $ =~ m/ (\d+) /xg;
 
 use Brick::Bucket;
 
@@ -11,8 +11,6 @@ package Brick::Bucket;
 use strict;
 
 use Carp qw(carp);
-use Storable qw(dclone);
-
 
 =head1 NAME
 
@@ -38,6 +36,8 @@ in programming rather than a failure of the data to validate.
 
 =over 4
 
+=item __and( LIST OF CODEREFS )
+
 =item __compose_satisfy_all( LIST OF CODEREFS )
 
 This is AND with NO short-circuiting.
@@ -57,6 +57,12 @@ sub __compose_satisfy_all
 	$bucket->__compose_satisfy_N( scalar @_, @_ );
 	}
 
+BEGIN {
+*__and = *__compose_satisfy_all;
+}
+
+=item __or( LIST OF CODEREFS )
+
 =item __compose_satisfy_any( LIST OF CODEREFS )
 
 This is OR but with NO short-circuiting.
@@ -75,6 +81,12 @@ sub __compose_satisfy_any
 	$bucket->__compose_satisfy_N_to_M( 1, scalar @_, @_ );
 	}
 
+BEGIN {
+*__or = *__compose_satisfy_any;
+}
+
+=item __none( LIST OF CODEREFS )
+
 =item __compose_satisfy_none( LIST OF CODEREFS )
 
 
@@ -94,6 +106,10 @@ sub __compose_satisfy_none
 	$bucket->__compose_satisfy_N_to_M( 0, 0, @_ );
 	}
 
+BEGIN {
+*__none = *__compose_satisfy_none;
+}
+
 =item __compose_satisfy_N( SCALAR, LIST OF CODEREFS )
 
 This function creates a new constraint that returns true if exactly N
@@ -127,7 +143,7 @@ sub __compose_satisfy_N_to_M
 		return sub {};
 		}
 
-	my @caller = main::__caller_chain_as_list();
+	my @caller = $bucket->__caller_chain_as_list();
 
 	my @composers = grep { /^__compose/ } map { $_->{sub} } @caller;
 
@@ -140,7 +156,7 @@ sub __compose_satisfy_N_to_M
 			my @dies = ();
 			foreach my $sub ( @subs )
 				{
-				eval { $sub->( @_ ) };
+				my $result = eval { $sub->( @_ ) };
 				my $at = $@;
 				$count++ unless $at;
 				#print STDERR "\n!!!!Sub died!!!!\n" if ref $at;
@@ -150,12 +166,14 @@ sub __compose_satisfy_N_to_M
 				};
 
 			my $range = $n == $m ? "exactly $n" : "between $n and $m";
-
+			
 			die {
 				message => "Satisfied $count of $max sub-conditions, needed to satisfy $range",
 				handler => $caller[0]{'sub'},
-				errors  => dclone \@dies,
+				errors  => \@dies,
 				} unless $n <= $count and $count <= $m;
+				
+			return 1;
 			},
 		});
 
@@ -164,6 +182,8 @@ sub __compose_satisfy_N_to_M
 	return $sub;
 	}
 
+=item __not( CODEREF )
+
 =item __compose_not( CODEREF )
  
 This composers negates the sense of the code ref. If the code ref returns
@@ -171,6 +191,7 @@ true, this composer makes it false, and vice versa.
 
 =cut
 
+
 sub __compose_not
 	{
 	my( $bucket, $not_sub ) = @_;
@@ -183,14 +204,18 @@ sub __compose_not
 	}
 
 
+=item __compose_until_pass
 
 =item __compose_pass_or_skip
 
-Go through the list of closures, trying each one until one suceeds. If a closure
-doesn't suceed, don't fail, just move on. Return true for the first one that
-passes, short-circuited the rest. If none of the closures pass, die with an
+Go through the list of closures, trying each one until one suceeds. If
+a closure doesn't die, but doesn't return true, this doesn't fail but
+just moves on. Return true for the first one that passes,
+short-circuited the rest. If none of the closures pass, die with an
 error noting that nothing passed.
 
+If one of the subs dies, this composer still dies.
+
 This can still die for programming (not logic) errors.
 
 =cut
@@ -205,7 +230,7 @@ sub __compose_pass_or_skip
 		return sub {};
 		}
 
-	my @caller = main::__caller_chain_as_list();
+	my @caller = $bucket->__caller_chain_as_list();
 
 	my $max = @subs;
 
@@ -231,7 +256,7 @@ sub __compose_pass_or_skip
 			die {
 				message => "Nothing worked! Unexpected failure of all branches",
 				handler => $caller[0]{'sub'},
-				errors  => dclone \@dies,
+				errors  => \@dies,
 				};
 			},
 		});
@@ -241,6 +266,12 @@ sub __compose_pass_or_skip
 	return $sub;
 	}
 
+BEGIN {
+*__compose_until_pass = *__compose_pass_or_skip;
+}
+
+=item __compose_until_fail
+
 =item __compose_pass_or_stop
 
 Keep going as long as the closures return true.
@@ -279,7 +310,7 @@ sub __compose_pass_or_stop
 		return sub {};
 		}
 
-	my @caller = main::__caller_chain_as_list();
+	my @caller = $bucket->__caller_chain_as_list();
 
 	my $max = @subs;
 
@@ -318,6 +349,10 @@ sub __compose_pass_or_stop
 	return $sub;
 	}
 
+BEGIN {
+*__compose_until_fail = *__compose_pass_or_stop;
+}
+
 =back
 
 =head1 TO DO
