@@ -112,6 +112,11 @@ came from the input to C<_input_key_exists>.
 	
 	my $dog_brick = $bucket->_input_key_exists( { field => 'dog' } );
 	
+Somewhere I defined the C<_input_key_exists> method so it shows
+up in the Bucket class:
+
+	package Brick::Bucket;
+	
 	sub _input_key_exists
 		{
 		my( $bucket, $setup ) = @_;
@@ -311,4 +316,6 @@ Copyright (c) 2007, brian d foy, All Rights Reserved.
 
 You may redistribute this under the same terms as Perl itself.
 
-=cut
\ No newline at end of file
+=cut
+
+1;
\ No newline at end of file
